@@ -161,7 +161,7 @@ setMethod("kurtosis", signature(x = "Exp"),
     if(hasArg(fun)||hasArg(cond)||!is.null(low)||!is.null(upp)) 
          return(kurtosis(as(x,"AbscontDistribution"),...))
     else
-        return(2)
+        return(6)
     })
  ### source http://mathworld.wolfram.com/ExponentialDistribution.html
 
@@ -460,3 +460,24 @@ setMethod("kurtosis", signature(x = "GPareto"),
     }
 })
 ### source Maple ...
+
+setMethod("kurtosis", signature(x = "GEV"),
+    function(x, ...){
+    dots <- match.call(call = sys.call(sys.parent(1)), 
+                       expand.dots = FALSE)$"..."
+    fun <- NULL; cond <- NULL; low <- NULL; upp <- NULL
+    if(hasArg(low)) low <- dots$low
+    if(hasArg(upp)) upp <- dots$upp
+    if(hasArg(fun)||hasArg(cond)||!is.null(low)||!is.null(upp)) 
+        return(kurtosis(as(x,"AbscontDistribution"),...))
+    else{
+         xi <- shape(x)
+         if(xi>=1/4) return(NA)
+         if(xi==0) return(12/5)
+         else
+         return((gamma(1-4*xi)- 4*gamma(1-xi)*gamma(1-3*xi)+6*gamma(1-2*xi)*gamma(1-xi)^2 - 3*gamma(1-xi)^4)/(gamma(1-2*xi)-gamma(1-xi)^2)^(2)) 
+    }
+})
+### source http://en.wikipedia.org/wiki/Generalized_extreme_value_distribution
+###        http://en.wikipedia.org/wiki/Gumbel_distribution
+###        http://en.wikipedia.org/wiki/Riemann_zeta_function 
